body {{ 
    color: #000;
    background-color: #fff;
}}
::-webkit-scrollbar {{
    background: silver;
}}
::-webkit-scrollbar-thumb {{
    background: gray;
}}
::-webkit-scrollbar-thumb:hover {{
    background: #000;
}}
::-webkit-scrollbar-corner {{
    background: {QTMATERIAL_SECONDARYLIGHTCOLOR};
}}
p {{ 
    color: #000;
}}
a {{
    color: #000;
}}
.msg-user {{
    color: #2c2c2c !important;
}}
.msg-user p {{
    color: #2c2c2c !important;
}}
.msg-bot {{
    color: #000;
}}
.msg-highlight {{
    border: 1px solid #dfdfdf;
    background-color: #eaeaea;
}}
.cmd {{
    color: #4d4d4d;
    text-decoration: none;
}}
.ts {{
    color: #4d4d4d;
}}
code {{
    color: #000;
}}
.code-wrapper {{
    border: 1px solid #e1e1e1;
    color: #fff;
}}
.code-header-wrapper {{
    color: #fff;
    background: #e1e1e1;
}}
.code-header-copy {{
    color: #444444;
}}
.code-header-copy:hover {{
    color: #000;
}}
.code-header-lang {{
    color: #5c5c5c;
}}
.display-blocks {{
    background-color: #e7e7e7;
}}
.display-blocks .msg-box {{
    background-color: #fff;
    border: 1px solid gray;
}}
.display-blocks .name-bot::before {{
    background-color: #6b6b6b;
}}
.display-blocks .name-user::before {{
    background-color: silver;
}}