* {{ 
  font-family: 'Lato' !important; 
}}
QWidget {{
  background-color: #e7e7e7;
  color: #000;
}}

QListView, 
QTreeView {{
  color: #000;
  background-color: #fff;
  border: 1px solid #c0c0c0;
}}
QListView::item:selected, 
QTreeView::item:selected,
QListView::item:selected:focus, 
QTreeView::item:selected:focus {{
  color: #fff;
  selection-color: {QTMATERIAL_PRIMARYTEXTCOLOR};
}}

QTabBar::tab:selected {{
  color: #000 !important;
}}
QTabBar::tab:top:selected,
QTabBar::tab:top:hover {{
  color: #000;
}}

QPushButton {{
  color: #3e3e3e;
  text-transform: none;
  background-color: {QTMATERIAL_SECONDARYCOLOR};
  border: 1px solid #fff;
}}

QLineEdit {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QLineEdit::focus {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QTextEdit {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QMenu {{ 
  background-color: #fff; 
}} 
QMenu::item {{ 
  color: #0e0e0e;
}} 
QComboBox,
QComboBox::focus {{
  background-color: #fff;
  color: #000000;
}}

QMainWindow::separator:horizontal,
QSplitter::handle:vertical {{
  width: 100%;
  height: 1px;
  background-color: #d4d4d4;
}}
QTableView::item,
QTreeView::item,
QListView::item {{
  color: #0e0e0e;
  padding: 4px;
  min-height: 20px;
}}
.label-title {{
  font-weight: bold;
}}
.label-help {{
  color: #414141;
}}
.label-chat-status {{
  color: #414141;
}}
.interpreter-input {{
  font-family: 'Monaspace Neon';
}}
.interpreter-output,
.code-editor {{
  font-family: 'Monaspace Neon';
  background: #fff; 
  color: #000;
}}
QCalendarWidget QWidget {{
  background: #fff;
}}
QScrollBar::handle {{
  background: {QTMATERIAL_PRIMARYCOLOR};
}}